/*
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#define _GNU_SOURCE

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>
#include <getopt.h>
#include <syslog.h>

#include <popt.h>
#include <glib.h>

#include <libhal/libhal.h>

#define MAX_CMD_LENGTH 2048

static int debug = FALSE;


int main (int argc, const char *argv[])
{
	int i;
	int ret_val = EXIT_FAILURE;
	poptContext popt_context;
    	char *udi = NULL;
	char *irq = NULL;
	char *port = NULL;
	char *input_dev = NULL;
	char cmd[MAX_CMD_LENGTH+1];
	const char **left_over_args = NULL;
	// DBusError error;

	struct poptOption options[] = {
	        {"udi",          's', POPT_ARG_STRING, &udi,       0, "udi of the related tablet device", "UDI"},
	        {"irq",          'i', POPT_ARG_STRING, &irq,       0, "irq of the tablet device", "PORT"},
	        {"port",         'p', POPT_ARG_STRING, &port,      0, "port of the tablet device", "IRQ"},
	        {"input-device", 'd', POPT_ARG_STRING, &input_dev, 0, "the input device to which should bind the tablet device", "INPUT_DEVICE"},

        	POPT_AUTOHELP
        	{NULL, '\0', 0, NULL, 0, NULL, NULL}
    	};

	popt_context = poptGetContext ("hal-tabletPC-setserial", argc, argv, options, 0);

	while ((i = poptGetNextOpt (popt_context)) != -1) {
		if (i < -1) {
			poptPrintHelp (popt_context, stderr, 0);
			ret_val = EXIT_FAILURE;
			goto early_out;
		}
	}

	left_over_args = poptGetArgs (popt_context);

	if (getenv ("HALD_VERBOSE") != NULL )
        	debug = TRUE;
	if (debug)
        	syslog (LOG_INFO, "hal-tabletPC-setserial started in debug mode." );

	if(udi == NULL)
		udi = getenv("UDI");
	if(irq == NULL)
		irq = getenv("HAL_PROP_PNP_SERIAL_IRQ");
	if(port == NULL)
		port = getenv("HAL_PROP_PNP_SERIAL_PORT");
	if(input_dev == NULL)
		input_dev = getenv("HAL_PROP_INPUT_DEVICE_SET");

	if (!udi || !irq || !port || !input_dev) {
		syslog (LOG_INFO, "Missing env variable, exit NOW." );
		goto early_out;
	}

	
	snprintf( cmd, MAX_CMD_LENGTH, "/bin/setserial %s port %s irq %s autoconfig", input_dev, port, irq);
	syslog (LOG_INFO, "Collected setserial options and Called(%d) %s ", system(cmd), cmd);

	ret_val = EXIT_SUCCESS;

early_out:
	poptFreeContext (popt_context);
	return ret_val;
}

